/*
This file is part of DashMan video game

Copyright (c) 2008 Michael Georgoulopoulos <msamurai@freemail.gr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// gfx.cpp
#include "gfx.h"

Rect::Rect(int left, int right, int top, int bottom)
{
	this->left = left;
	this->right = right;
	this->top = top;
	this->bottom = bottom;
}

Color::Color(float r, float g, float b, float a)
{
	this->r = r;
	this->g = g;
	this->b = b;
	this->a = a;
}



// Dev-Cpp sucks!
// I have to rewrite glOrtho2D here ....
void MultOrtho(GLfloat left, GLfloat right,
		GLfloat bottom, GLfloat top,
		GLfloat nearval, GLfloat farval)
{
    GLfloat m[16];
#define M(row,col)  m[col*4+row]

	M(0,0) = 2.0F / (right-left);
	M(0,1) = 0.0F;
	M(0,2) = 0.0F;
	M(0,3) = -(right+left) / (right-left);

	M(1,0) = 0.0F;
	M(1,1) = 2.0F / (top-bottom);

	M(1,2) = 0.0F;
	M(1,3) = -(top+bottom) / (top-bottom);

	M(2,0) = 0.0F;
	M(2,1) = 0.0F;
	M(2,2) = -2.0F / (farval-nearval);
	M(2,3) = -(farval+nearval) / (farval-nearval);

	M(3,0) = 0.0F;
	M(3,1) = 0.0F;
	M(3,2) = 0.0F;
	M(3,3) = 1.0F;
#undef M
	glMultMatrixf(m);
}

void Blit(const Rect &src_rect, const Rect &dst_rect, const Color &color)
{
    // calculate texture coordinates
    float half_texel = 0.5f / 512;
    float u0 = (float)src_rect.left / 512 + half_texel;
    float u1 = (float)src_rect.right / 512 - half_texel;
    float v0 = (float)src_rect.top / 512 + half_texel;
    float v1 = (float)src_rect.bottom / 512 - half_texel;

    glBegin(GL_QUADS);
    {
        glColor4f(color.r, color.g, color.b, color.a);

        glTexCoord2f(u0, v0);
        glVertex2f(dst_rect.left, dst_rect.top);

        glTexCoord2f(u1, v0);
        glVertex2f(dst_rect.right, dst_rect.top);

        glTexCoord2f(u1, v1);
        glVertex2f(dst_rect.right, dst_rect.bottom);

        glTexCoord2f(u0, v1);
        glVertex2f(dst_rect.left, dst_rect.bottom);
    }
    glEnd();
}

void Blit(int xframe, int yframe, int xpos, int ypos, const Color &color)
{
    Rect src = Rect(xframe * 32, xframe * 32 + 32, yframe * 32, yframe * 32 + 32);
    Rect dst = Rect(xpos, xpos + 32, ypos, ypos + 32);
    Blit(src, dst, color);
}

void BlitString(int x, int y, int spacing, const char *str, const Color &color)
{
    char *s = (char*)str;
    int cx = x;
    while (*s)
    {
        char c = *s;
        // convert to uppercase
        if (c >= 'a' && c <= 'z') c -= 'A' - 'a';
        int xfr, yfr;
        if (c >= 'A' && c <= 'P')
        {
            xfr = c - 'A';
            yfr = 4;
        }
        else if (c >= 'Q' && c <= 'Z')
        {
            xfr = c - 'Q';
            yfr = 5;
        }
        else if (c >= '1' && c <= '9')
        {
            xfr = c - '1';
            yfr = 6;
        }
        else if (c == '0')
        {
            xfr = 9;
            yfr = 6;
        }
        else
        {
            // print blank
            xfr = 10;
            yfr = 5;
        }

        Blit(xfr, yfr, x + cx, y, color);
        cx += spacing;
        s++;
    }
}
