/*
This file is part of DashMan video game

Copyright (c) 2008 Michael Georgoulopoulos <msamurai@freemail.gr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <stdio.h>
#include <stdlib.h>

#include <SDL.h>

#include "load_png.h"
#include "gfx.h"
#include "game.h"

#include <math.h>

//#include "FMOD/fmod.h"
#include "sound.h"

static int xres = 640;
static int yres = 480;
int frames = 0;
int processed_frames = 0;
unsigned int texture = 0;
unsigned int start_time = 0;

// mapped user controls
Direction user_dir;
bool user_action = false;

//FSOUND_STREAM *music;

bool Init()
{
    if (!LoadLevels()) return false;

	Image img;
	if (LoadImageFromFile("data/bank.png", &img))
	{
		printf("failed to load bitmap bank\n");
		return false;
	}

	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER , GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER , GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, img.pixels);

	DeleteImage(&img);

	glDepthFunc(GL_ALWAYS);
	glEnable(GL_LINE_SMOOTH);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	MultOrtho(-0.5f, 640, 480, -0.5f, -1, 1);
	
	// Init fmod
	/*
	if (!FSOUND_Init(44100, 32, 0))
	{
        return false;
    }
	*/
	if (!LoadSounds())
	{
		return false;
	}
	
	// start music
	/*
	music = FSOUND_Stream_Open(
		"data/sounds/music.mp3",
		FSOUND_NORMAL | FSOUND_LOOP_NORMAL,
		0, 0);

    FSOUND_Stream_Play(FSOUND_FREE, music);
	*/

	return true;
}

void Cleanup()
{
	DeleteSounds();
    //FSOUND_Close();
}

bool ProcessEvents()
{
    user_dir = NODIR;
    user_action = false;
    SDL_Event event;
    
    SDL_PollEvent(&event);
   
    if(event.key.keysym.sym == SDLK_ESCAPE)
        return false;
      
    if (event.key.type == SDL_KEYDOWN)
    {
        if(event.key.keysym.sym == SDLK_UP)
            user_dir = UP;
        if(event.key.keysym.sym == SDLK_DOWN)
            user_dir = DOWN;
        if(event.key.keysym.sym == SDLK_LEFT)
            user_dir = LEFT;
        if(event.key.keysym.sym == SDLK_RIGHT)
            user_dir = RIGHT;
    }
    
    if (event.key.state == SDL_RELEASED)
    {
        if(event.key.keysym.sym == SDLK_RETURN)
            user_action = true;
    }
    
    return true;
}

void Render()
{
    glClearColor(0.4f, 0.4f, 0.4f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glBindTexture(GL_TEXTURE_2D, texture);

    DrawLevel();

	glFlush();
	glFinish();
}

int main(int argc, char *argv[])
{
	int i;
	bool fullscreen = true;

	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-' && argv[i][2] == 0) {
			switch(argv[i][1]) {
			case 'w':
				fullscreen = false;
				break;

			default:
				fprintf(stderr, "unexpected argument: %s\n", argv[i]);
				return 1;
			}
		} else {
			fprintf(stderr, "unexpected argument: %s\n", argv[i]);
			return 1;
		}
	}

    SDL_Init(SDL_INIT_VIDEO);
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    SDL_SetVideoMode(640, 480, 32, SDL_OPENGL | (fullscreen ? SDL_FULLSCREEN : 0));
    SDL_ShowCursor(0);

	glViewport(0, 0, 640, 480);
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glDisable(GL_DEPTH_TEST);
    glShadeModel(GL_SMOOTH);
  
    if (!Init()) return 0;


    start_time = SDL_GetTicks();
    while (ProcessEvents())
    {
        unsigned int time = SDL_GetTicks() - start_time;
        frames = (time * 25) / 1000;

        if (processed_frames < frames)
        {
            if (!UpdateGame(user_dir, user_action)) break;
            user_dir = NODIR;
            user_action = false;
            processed_frames++;
        }
        Render();
        SDL_GL_SwapBuffers();
    }
    SDL_Quit();
  
    return 0;
}
